==> trip-0.1.0.tar.gz: trip-0.1.0/tripstack/tripcore/TRIPItadTopology.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPItadTopology_cxx_Version = 
"$Id: TRIPItadTopology.cxx,v 1.2 2000/11/13 21:53:30 kevin Exp $";

#ifdef __linux__
    #include <algo.h>
#else
    #include <algorithm>
#endif

#include <stdlib.h>

#include "TRIPItadTopology.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"


TRIPItadTopology::TRIPItadTopology(const TRIPItadTopology& other) 
{
     copyObj(other);
}

TRIPItadTopology& 
TRIPItadTopology::operator=(const TRIPItadTopology& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPItadTopology::dump(ostream& os) const
{
    os.width(25);
    os << "**ItadTopology**" << endl;
    os.width(25);
    int idx = 0;
    for(TripIdList::const_iterator itr = _tripIdList.begin();
               itr != _tripIdList.end(); itr++)
    {
        os << (*itr) << ",";
        idx++;
        if(idx > 5)
        {
            //Print no more than 5 IDs in a line
            os << endl;
            os.width(25);
            idx = 0;
        }
    }
}

void 
TRIPItadTopology::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    assert(_tripIdList.size());
    for(TripIdList::const_iterator itr = _tripIdList.begin();
               itr != _tripIdList.end(); itr++)
    {
        (*itr).writeData(id);
    }
}

TRIPItadTopology::TRIPItadTopology(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    int hdrLen  = _hdr->getHdrLength(); 
    int len = _hdr->getLength();

    if(_hdr->getAttrType() != TRIP_ITAD_TOPOLOGY)
    {
        cpLog(LOG_ALERT, "Bad ITAD topology attribute");
        throw TRIPBadDataException("Bad ITAD topology attribute",
                                   __FILE__, __LINE__, 0);
    }

    u_int16_t tLen = 0;
    while(tLen < len)
    {
        TRIPId tId(buf+hdrLen+tLen);
        tLen += tId.getLength();
        _tripIdList.push_back(tId);
    }
}

void 
TRIPItadTopology::copyObj(const TRIPItadTopology& other)
{
    TRIPRouteAttr::copyObj(other);
    _tripIdList.clear();
    for(TripIdList::const_iterator itr = other._tripIdList.begin();
               itr != other._tripIdList.end(); itr++)
    {
        _tripIdList.push_back(*itr);
    }
}

void 
TRIPItadTopology::addTripId(const TRIPId& tripId)
{
    TripIdList::const_iterator itr = find(_tripIdList.begin(), _tripIdList.end(), tripId);
    if(itr == _tripIdList.end())
    {
        _tripIdList.push_back(tripId);
    }
    setLength(tripId.getLength() * _tripIdList.size());
}

==> trip-1.0.0.tar.gz: trip-1.0.0/tripstack/tripcore/TRIPItadTopology.cxx <==
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF $1,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPItadTopology_cxx_Version = 
"$Id: TRIPItadTopology.cxx,v 1.9 2001/03/05 20:21:19 kevinmc Exp $";

#include <netinet/in.h>
#include <stdlib.h>

#include <algorithm>
#include "TRIPItadTopology.hxx"
#include "TRIPUtil.hxx"
#include "TRIPErrorHandler.hxx"
#include "cpLog.h"


TRIPItadTopology::TRIPItadTopology(const TRIPId& id, u_int32_t seqNum, u_int8_t flg)
            : TRIPRouteAttr(flg, TRIP_ITAD_TOPOLOGY, id, seqNum)
{
    setLength(0);
}

TRIPItadTopology::TRIPItadTopology(const TRIPItadTopology& other) 
{
     copyObj(other);
}

TRIPItadTopology& 
TRIPItadTopology::operator=(const TRIPItadTopology& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPItadTopology::dump(ostream& os) const
{
    os.width(DUMP_LABEL_WIDTH);
    os << "**ItadTopology**" << endl;
    os.width(DUMP_LABEL_WIDTH);
    int idx = 0;
    for(TripIdList::const_iterator itr = _tripIdList.begin();
               itr != _tripIdList.end(); itr++)
    {
        os << (*itr) << ",";
        idx++;
        if(idx > 5)
        {
            //Print no more than 5 IDs in a line
            os << endl;
            os.width(DUMP_LABEL_WIDTH);
            idx = 0;
        }
    }
}

void 
TRIPItadTopology::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    assert(_tripIdList.size());
    for(TripIdList::const_iterator itr = _tripIdList.begin();
               itr != _tripIdList.end(); itr++)
    {
        (*itr).writeData(id);
    }
}

TRIPItadTopology::TRIPItadTopology(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    assert(buf);

    if(_hdr->getAttrType() != TRIP_ITAD_TOPOLOGY)
    {
        cpLog(LOG_ALERT, "Bad ITAD topology attribute");
        throw TRIPBadDataException("Bad ITAD topology attribute",
                                   __FILE__, __LINE__, TRIP_ERR_UPDATE);
    }

    int hdrLen  = _hdr->getHdrLength(); 
    int len = _hdr->getLength();
    assert(len);
    u_int16_t tLen = 0;
    //Loop through the length of the attribute, extract the TRIPIds and
    //build the ID list.
    while(tLen < len)
    {
        TRIPId tId(buf+hdrLen+tLen);
        tLen += tId.getLength();
        _tripIdList.push_back(tId);
    }
}

void 
TRIPItadTopology::copyObj(const TRIPItadTopology& other)
{
    TRIPRouteAttr::copyObj(other);
    _tripIdList.clear();
    for(TripIdList::const_iterator itr = other._tripIdList.begin();
               itr != other._tripIdList.end(); itr++)
    {
        _tripIdList.push_back(*itr);
    }
    setLength(TRIPId::getLength() * _tripIdList.size());
}

void 
TRIPItadTopology::addTripId(const TRIPId& tripId)
{
    TripIdList::const_iterator itr = find(_tripIdList.begin(), _tripIdList.end(), tripId);
    if(itr == _tripIdList.end())
    {
        _tripIdList.push_back(tripId);
        setLength(TRIPId::getLength() * _tripIdList.size());
    }
}


bool
TRIPItadTopology::isEqualAttribute(const TRIPRouteAttr& attr) const
{
    assert(attr.getType() == TRIP_ITAD_TOPOLOGY);
    const TRIPItadTopology* other = dynamic_cast<const TRIPItadTopology*>(&attr);
    assert(other);
    if(!other) return false;
    return (TRIPRouteAttr::isEqualAttribute(attr) &&
            (_tripIdList == other->_tripIdList));
}
